defmodule Chronoscope.NTS.CertificateTest do
  use ExUnit.Case

  import Chronoscope.NTS.Certificate

  test "parses the expiration date of a certificate" do
    {:ok, expiration, _} =
      :secp256r1
      |> X509.PrivateKey.new_ec()
      |> X509.Certificate.self_signed("/C=US/ST=CA/L=San Francisco/O=Acme/CN=Test", validity: 12)
      |> X509.Certificate.to_der()
      |> expiration_date()
      |> DateTime.from_iso8601()

    assert DateTime.diff(expiration, DateTime.utc_now(), :day) == 12
  end

  # todo - mock current time
  test "converts certificate datetime to iso8601" do
    assert cert_time_to_iso8601("240326110000Z") == "2024-03-26T11:00:00Z"
  end

  test "handles century rollover" do
    assert cert_time_to_iso8601("010326110000Z") == "2101-03-26T11:00:00Z"
  end
end
